//bugf84
//This is the driver program for the Cyber Spider.  It will use a 754410
//driver for the two motors, a TTT IRPD board for object detection and 
//will generate a PWM signal for the motor drivers.  Eventually it will
//use LDRs to sense light and dark.  Its designed to use the default IRQ
//handler (very slow) and a 4MHZ 16C84 (also very slow) so the PWM 
//resolution is only 4 bits (16 values in each direction).  Wimpy, but
//more than adequate for this project!
//
//Version 1.0 Copyright Dennis Clark and TTT Enterprises February 2002
//Version 2.0 (c) Dennis Clark and TTT Enterprises January 2009 for 16F628

//#include <16F84.H>
#include <16F628.H>

#fuses HS,NOWDT,NOPROTECT,NOPUT

#use delay(clock=4000000)
#use FAST_IO(A)						//don't keep setting TRIS register
#use FAST_IO(B)
#byte PORTB = 6					//writing bits to 754410 direction
#byte PORTA = 5					//IRPD on low bits, PWM on high bits
#define RTL 0x65					//rotate left
#define RTR 0x6A					//rotate right
#define REV 0x69					//go reverse
#define FWD 0x66					//go forward
#define D_ON 0x2F					//downlook on
#define I_ON 0x4F					//IRPD on
#define B_ON 0x6F					//Both on
#define WANDER 1				//Behavior priorities
#define AVOID  2
//#define BUMPED 3
#define EDGE  4

const byte SLOW = 6;				//slow motor speed
const byte MED = 10;
const byte FAST = 15;

byte IRPDMask;						//masking off bits controlling IRPDs
byte currDir;						//current direction bits

byte left;    						// current left PWM count
byte right;							// current right PWM count
byte PWMperiod;						// to get full 1KHz period

byte sleft;							//left PWM setting
byte sright;						//right PWM setting
const byte SPERIOD = 15;			//16 passes = 1KHz PWM

//Globals
long dur;							//how long to go that way
byte priority;						//currently active priority

#int_rtcc                          	// This function is called every time
void clock_isr(void) 
{                      				// the RTCC (timer0) overflows (255->0).
    if (left == 0) 
	{
        output_bit(PIN_A2,0);
    }
    left--;
    if (right == 0) 
	{
        output_bit(PIN_A3,0);
    }
    right--;
    if (PWMperiod == 0) 
	{
        PWMperiod = SPERIOD;
        left = sleft;
        right = sright;
        output_bit(PIN_A2,1);
        output_bit(PIN_A3,1);
    } 
    PWMperiod--;					//C int maint makes this > 32us!
    set_rtcc(240);  				//255-239 =16, 16*2us = 32us
}

//behaviors

void do_wander(void) 
{
	//Does just the wander routine

     if (priority == WANDER) {
        dur--;
        if (dur == 0)
            priority = 0;
    }
    if (priority < WANDER) {
        priority = WANDER;
        currDir = FWD;
        PORTB = currDir;
        sleft = MED;
        sright = MED;
        dur = 100;
    } 
}

void do_avoid(void) 
{
	//Does the object avoidance

    byte irpd;									//IRPD inputs

    if (priority == AVOID) 
	{
		dur--;
		if (dur == 0)
		{
            priority = 0;
		}
    }
    if (priority < AVOID) 
	{
		currDir = FWD;
		PORTB = currDir;				//Note IRPD bits 
        irpd = PORTA;
        irpd &= 0x03;
        if (irpd != 0) 
	    {
	    	priority = AVOID;
            switch (irpd) 
	        {
	    	case 3: // directly ahead
				currDir = REV;
				PORTB = currDir;
				sleft = FAST;
                sright = SLOW;
				dur = 3000;
				break;
	    	case 2: // something to the right
				currDir = RTL;
				PORTB = currDir;
				sleft = FAST;
				sright = FAST;
				dur =300;
				break;
	   		case 1: // something to the left
				currDir = RTR;
				PORTB = currDir;
				sleft = FAST;
				sright = FAST;
				dur = 300;
				break;
	    	}
        } 
    }
}

void do_edge(void) 
{
	// Watches for the edge of the table
    byte edet;										//edge detector

    if (priority == EDGE) 
	{
		dur--;
		if (dur < 6000)
		{
			currDir = RTR;
			PORTB = currDir;
		}
		if (dur == 0) 
		{
            priority = 0;
		}
    }
    if (priority < EDGE) 
	{
        edet = PORTB;
        edet &= 0x10;
        if (edet) 								//1 == edge of the world
	    {		
	    	priority = EDGE;
	    	currDir = REV;
	    	PORTB = currDir;
	    	sleft = FAST;
            sright = FAST;
	    	dur = 9000;
		}
    }
}

void Init(void)
{
	// This starts it all off, set up the ports, IRQ and defaults
	/*
	PORTA 0 = Left IRPD
		  1 = Right IRPD
		  2 = Left PWM
		  3 = Right PWM
		  4 = N/C
		  5 = N/C
		  
	PORTB 0 = 1A Right Motor direction pin
		  1 = 2A Right Motor direction pin
		  2 = 3A Left Motor direction pin
		  3 = 4A Left Motor direction pin
		  4 = Down Sharp GP2Y0D340K
		  5 = Down IRPD enable N/C
		  6 = IRPD enable N/C
		  7 = N/C
	*/

    set_tris_A(0x03);							//0,1 inputs, 2-4 outputs
    
	setup_comparator(NC_NC_NC_NC);				//turn comparator off
	
    //Set up PWM interrupt stuff
    setup_timer_0( RTCC_INTERNAL | RTCC_DIV_2);
    enable_interrupts(INT_TIMER0);
    enable_interrupts(GLOBAL);
    set_timer0(240);
    
    set_tris_B(0x90);							//0-3 outputs, 4&7 inputs, 5&6 outputs
}
	
void main(void) 
{    
    Init();
    
    sleft=0;									//left legs speed 0-15
    sright = 0;									//right legs speed
    PWMperiod = SPERIOD;   						//overall period (about 1ms)

    currDir = FWD;
    PORTB = currDir;
    sleft = 0;
    sright = 0;

    priority = 0;								// no initial behavior

    delay_ms(1000);								// Lets wait for a few seconds first

    while(1) 
	{
        do_wander();
        do_avoid();
        do_edge();
    }

}


